@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2005  The PARI group.
+Copyright (C) 2000-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,7 +10,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include "config.h"
 #include <stdlib.h>
@@ -102,7 +102,10 @@ int guesstype(int n)
       return Gstr;
     }
   case Fentry:
-    return vartype(ctxstack[getvarerr(n)]);
+    {
+      long v = getvar(n);
+      return v<0 ? Ggen: vartype(ctxstack[v]);
+    }
   case Ffunction:
     if (!strcmp(entryname(n),"length"))
       return Glg;
@@ -133,17 +136,30 @@ void genequaltosmall(int binf,int bsup, int *tv, int def)
 
 int gengerepile(int bl)
 {
-  int arg[4];
+  int arg[3];
   block[bl].gc|=(1<<GCneeded);
   arg[0]=newnode(Fentry,newentry("btop"),-1);
   pushvar(arg[0],0,Gpari_sp,newcall("_avma",-1));
-  arg[1]=newnode(Fentry,newentry("st_lim"),-1);
-  pushvar(arg[1],0,Gpari_sp,
-      newcall("_stack_lim",newnode(Flistarg,arg[0],newsmall(1))));
-  arg[2]=newsmall(bl);
+  arg[1]=newsmall(bl);
+  if (FC_gc_needed>=0)
+    arg[2]=GNOARG;
+  else
+  {
+    arg[2]=newnode(Fentry,newentry("st_lim"),-1);
+    pushvar(arg[2],0,Gpari_sp,
+        newcall("_stack_lim",newnode(Flistarg,arg[0],newsmall(1))));
+  }
   return stacktoargs(arg,3);
 }
 
+int gengerepilereturn(int bl)
+{
+  int arg[2];
+  arg[0]=newnode(Fentry,newentry("ltop"),-1);
+  arg[1]=newsmall(bl);
+  return stacktoargs(arg,2);
+}
+
 int newgetg(int v, int l, const char *t)
 {
   return newcall("_cgetg", newnode(Flistarg, newnode(Flistarg,
@@ -153,8 +169,10 @@ int newgetg(int v, int l, const char *t)
 void makesubblock_ctx(int n, int savx)
 {
   int bl3=newblock();
+  int level = preclevel;
   genblock(n,-1);
   makeblock(bl3,n,newleaf(n),-1,savx);
+  preclevel = level;
 }
 
 void makesubblock(int n)
@@ -258,8 +276,11 @@ geniterproto(int n, long fl, gpfunc *gp, int *argo, int *seq, int *var, int *tv)
       break;
     case PPdefault:
     case PPdefaultmulti:
-      if (i<nb) tree[arg[i]].t = guesstype(arg[i]);
-      argo[k++] = i<nb ? arg[i] : GNOARG;
+      if (i < nb)
+      {
+        tree[arg[i]].t = guesstype(arg[i]);
+        argo[k++] = arg[i];
+      }
       break;
     case PPstar:
       i=nb-1;
@@ -276,13 +297,13 @@ geniterproto(int n, long fl, gpfunc *gp, int *argo, int *seq, int *var, int *tv)
 }
 
 void
-genblockfunciter(int n, int p, gpfunc *gp)
+genblockfunciter(int n, gpfunc *gp)
 {
   int arg[STACKSZ];
   int var=-1, tv=Gnotype;
   int bseq=-1,aseq=-1,ret=-1,vprime,seq,cseq;
-  int gpm=-1,bl1,bl2;
-  int savx;
+  int gpm=-1,bl0,bl1,bl2;
+  int sav0, savx;
   Clabel_t lab;
   gpiterator * iter;
   const char * init, *next;
@@ -292,6 +313,8 @@ genblockfunciter(int n, int p, gpfunc *gp)
   iter = &(da->iter);
   init = lfunc[iter->init].gpname;
   next = lfunc[iter->next].gpname;
+  sav0=s_ctx.n;
+  bl0=newblock();
   vprime=newnode(Fentry,newentry("iter"),-1);
   pushvar(vprime,1<<Cuser,iter->type,-1);
   nb = geniterproto(n, 0, gp, arg+1, &seq, &var, &tv)+1;
@@ -320,7 +343,9 @@ genblockfunciter(int n, int p, gpfunc *gp)
   aseq=addseqright(aseq,newcall("_whilenull",newnode(Flistarg,cseq,seq)));
   if (lab.go)
     aseq=newseq(aseq,newcall("_label",newsmall(lab.go)));
-  makeblocks(bl1,bl2,n,bseq,aseq,ret,savx);
+  makeblocks3(bl0,bl1,bl2,n,bseq,aseq,ret,savx,savx);
+  copyctx(sav0,block+bl0);
+  restorectx(sav0);
 }
 
 /*
@@ -415,8 +440,11 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
         arg[0] = newleaf(pred);
       }
       genblock(arg[0],n);
-      arg[1]=geninsertvar(arg[1],ret);
-      makesubblock(arg[1]);
+      if (arg[1]!=GNOARG)
+      {
+        arg[1]=geninsertvar(arg[1],ret);
+        makesubblock(arg[1]);
+      }
       if(nb==3)
       {
         arg[2]=geninsertvar(arg[2],ret==-1?-1:newleaf(ret));
@@ -470,6 +498,9 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
       case 'l':
         functype(*gp)=Gsmall;
         break;
+      case 'u':
+        functype(*gp)=Gusmall;
+        break;
       case 'i':
         functype(*gp)=Gsmall_int;
         break;
@@ -483,7 +514,7 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
       }
       if (tree[arg[1]].f==Ftag)
         functype(*gp)=tree[arg[1]].y;
-      for(;*p;p++) if (*p=='p') funcmode(*gp)|=(1<<Mprec);
+      for(;*p;p++) if (*p=='p' || *p=='b') funcmode(*gp)|=(1<<Mprec);
       gp->spec=GPinstalled;
     }
     break;
@@ -623,15 +654,20 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
         sarg[0] = newnode(Fassign,newleaf(vindex),newsmall(1));
         sarg[1] = newleaf(lstep);
         sarg[2] = newnode(Fmatcoeff,newleaf(vstep),
-            newnode(Fmatrix,newnode(Frange,newleaf(vindex),GNOARG),-1));
+            newnode(Fmatrix,newnode(Frange,newleaf(vindex),GNORANGE),-1));
         sstep = newcall("sum", stacktoargs(sarg,3));
+        if (tv==Gnotype) tv=Ggen;
       }
       else
       {
         isvec=0;
         dosign=tree[bstep].f!=Fsmall;
         if (dosign)
+        {
           sstep = newleaf(vstep);
+          if (tv==Gnotype) tv=Ggen;
+        } else
+          genequaltosmall(binf,bsup,&tv,Ggen);
       }
       if (dosign)
       {
@@ -645,8 +681,6 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
       bl2=newblock();
       if (autogc)
         gpm=gengerepile(bl2);
-      if (tv==Gnotype)
-        tv=Ggen;
       pushvar(var,1<<Cuser,tv,-1);
       lab=makesubblock_label(seq,Lbreak);
       if (lab.ne)
@@ -950,6 +984,37 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
       if (tree[p].f==Ftag) tree[p]=tree[n];
     }
     break;
+  case GPself:
+    {
+      int str = newstringnode(lfunc[currfunc].gpname,-1);
+      int aseq = newcall("_strtoclosure",str);
+      tree[n] = tree[aseq];
+    }
+    break;
+  case GPlocalprec:
+    {
+      char buf[64];
+      int vn;
+      sprintf(buf,"prec%d",++preclevel);
+      vn = newnode(Fentry, newentry(strdup(buf)), -1);
+      pushvar(vn, (1<<Cuser), Gsmall, -1);
+      y = newnode(Fassign, vn, newcall("_ndec2prec", y));
+      genblock(y, n);
+      tree[n].y = y;
+    }
+    break;
+  case GPlocalbitprec:
+    {
+      char buf[64];
+      int vn;
+      sprintf(buf,"bitprec%d",++preclevel);
+      vn = newnode(Fentry, newentry(strdup(buf)), -1);
+      pushvar(vn, (1<<Cuser), Gsmall, -1);
+      y = newnode(Fassign, vn, newcall("_ndec2nbits", y));
+      genblock(y, n);
+      tree[n].y = y;
+    }
+    break;
   case GPreturn:
     {
       int iferr_reset=0;
@@ -987,10 +1052,7 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
       block[bl2].egc=var;
       if (autogc)
       {
-        arg[0]=newnode(Fentry,newentry("ltop"),-1);
-        arg[1]=GNOARG;
-        arg[2]=newsmall(bl2);
-        gpm=stacktoargs(arg,3);
+        gpm=gengerepilereturn(bl2);
         aseq=addseqright(aseq,newcall("_gerepilemany",gpm));
       }
       if (iferr_reset)
@@ -1055,6 +1117,7 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
       if (p>=0 && tree[p].f!=Fseq)
         newdecl((1<<Cauto)|(1<<Cimmutable),Gvoid,GNIL,&ret);
       seq=geninsertvar(seq,ret);
+      bseq=geninsertvar(bseq,ret);
       lab=makesubblock_label(seq,Liferr);
       savx=s_ctx.n;
       pushvar(var,1<<Cuser,tv,newcall("_iferr_error",-1));
@@ -1479,8 +1542,8 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     }
     break;
   case GPgpmany:
-    nb=genlistargs(n,arg,3,STACKSZ);
-    bl=block+tree[arg[2]].x;
+    nb=genlistargs(n,arg,2,3);
+    bl=block+tree[arg[1]].x;
     if (!(bl->gc&(1<<GCneeded)))
       break;
     genindent(fout);
@@ -1505,11 +1568,19 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     {
       int parg[STACKSZ+1];
       /*FIXME:Putting an if statement in the wilderness is not secure*/
-      if (arg[1]!=GNOARG)
+      if (nb==3)
       {
-        parg[0]=arg[1]; parg[1]=arg[0];
         fprintf(fout,"if (");
-        genfuncbydesc(fout,2,parg,FC_low_stack_lim,n);
+        if (arg[2]==GNOARG)
+        {
+          parg[0]=arg[0];
+          genfuncbydesc(fout,1,parg,FC_gc_needed,n);
+        }
+        else
+        {
+          parg[0]=arg[2]; parg[1]=arg[0];
+          genfuncbydesc(fout,2,parg,FC_low_stack_lim,n);
+        }
         fprintf(fout,")\n");
         indent++;
         genindent(fout);
@@ -1522,6 +1593,11 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
       fprintf(fout,";\n");
     }
     break;
+  case GPlocalprec:
+  case GPlocalbitprec:
+    gencode(fout, y);
+    preclevel++;
+    break;
   case GPreturn:
     fprintf(fout,"return");
     if (functype(lfunc[currfunc])!=Gvoid)
