@@ -1,16 +1,19 @@
 /*
-Copyright (C) 2000-2013  The PARI group.
+Copyright (C) 2000-2015  The PARI group.
 
 This file is part of the GP2C package.
 
-PARI/GP is free software; you can redistribute it and/or modify it under the
-terms of the GNU General Public License as published by the Free Software
-Foundation. It is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY WHATSOEVER.
+GP2C is free software; you can redistribute it and/or modify it under the terms
+of the GNU General Public License as published by the Free Software Foundation;
+either version 2 of the License, or (at your option) any later version.
 
-Check the License for details. You should have received a copy of it, along
-with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+This program is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
+Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #define TYPE
 #include "config.h"
@@ -22,32 +25,23 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include "header.h"
 
 FILE *yyin;
-int yyparse (void);
-void  yyrestart(FILE *new_file);
-void patchfunclist(void);
-void inittype(void);
-void initoperators(void);
 int linecount;
 int yydebug;
 static int optgen=0,opttree=0,opttype=0,optinfo=0;
 int optstrict=0;
 int optcleanvar=0;/*used in genbrace to suppress {} optimization*/
+int opthashline=0;
 int indentlevel=2;
+int preclevel=0;
 int do_warning=1;
 char *optprefix,*optsuffix;
 int debug;
-const char *nameparse, *namelib;
+const char *nameparse, *namelib, *outfile;
 int lastpass;
 int autogc;
 int warn;
 int safecoeff;
 
-void printlistfunc(FILE *fout);
-void outputtype(FILE *fout);
-void cleancode(int n, int p);
-void cleanvar(int n);
-void initdesc(const char *s);
-void maketreeGRL(FILE *fout,int n);
 void dump(FILE *fout)
 {
   int i,j;
@@ -99,6 +93,8 @@ init_compiler(void)
   n=newnode(Fnoarg,-1,-1); tree[n].t=Gvoid;
    /*Node 1 should be a (Gnotype)Fnoarg*/
   n=newnode(Fnoarg,-1,-1); tree[n].t=Gnotype;
+   /*Node 2 should be a Fnorange*/
+  n=newnode(Fnorange,-1,-1); tree[n].t=Gsmall;
 }
 
 void
@@ -122,6 +118,7 @@ init_stdfunc(void)
   FC_avma=findfuncdesc("_avma");
   Gpari_sp=findfunctype("_avma");
   FC_low_stack_lim=findfuncdesc("_low_stack_lim");
+  FC_gc_needed=findfuncdescopt("_gc_needed");
   FC_gerepileall=findfuncdesc("_gerepileall");
   FC_forprime_init=findfuncdescopt("_forprime_init");
   if (FC_forprime_init>=0)
@@ -144,6 +141,7 @@ init_stdfunc(void)
     warning(-1,"gp2c option -C not supported by this PARI version");
     safecoeff=0;
   }
+  FC_norange=findfuncdescopt("_norange");
 }
 
 void
@@ -166,14 +164,17 @@ compile(FILE *fin, FILE *fout, const char *nom)
 {
   const char *descfile;
   char *tmplib;
+  const char *lib;
   int startnode;
   int status;
   int i;
   init_compiler();
   linecount=1;
   nameparse=nom;
-  tmplib=(char*)calloc(strlen(nameparse)+6,sizeof(*nameparse));
-  sprintf(tmplib,"%s%s.so",nameparse[0]=='/'?"":"./",nameparse);
+  lib = genoutfile();
+  tmplib=(char*)calloc(strlen(lib)+6,sizeof(*lib));
+  sprintf(tmplib,"%s%s.so",lib[0]=='/'?"":"./",lib);
+  free((void*)lib);
   namelib=tmplib;
   yyin=fin;
   currfunc=-1;
@@ -218,7 +219,7 @@ compile(FILE *fin, FILE *fout, const char *nom)
   if (yydebug) fprintf(stderr,"End of parsing\n");
 
   startnode=addseqleft(newnode(Ffunction,newentry("_initfunc"),-1),startnode);
-  gentoplevel(startnode,-1);
+  gentoplevel(startnode);
   startnode=addinitfunc(startnode);
   if (debug>1)
     printnode(stderr,startnode);
@@ -270,7 +271,7 @@ compile(FILE *fin, FILE *fout, const char *nom)
   if (optinfo) dump(stderr);
   if (!optcleanvar)
     cleanvar(startnode);
-  cleancode(startnode,-1);
+  cleancode(startnode,-1,left);
   if (tree[startnode].f==Fseq && tree[startnode].y==GNIL)
     startnode=tree[startnode].x;
   switch(opttree)
@@ -294,7 +295,7 @@ compile(FILE *fin, FILE *fout, const char *nom)
 void version(void)
 {
   printf("GP to C compiler version %s \n   targeted at PARI/GP %s\n",VERSION,PARI_VERSION);
-  printf("Copyright 2000-2013 The PARI Group\n");
+  printf("Copyright 2000-2015 The PARI Group\n");
   printf("GP2C is free software, covered by the GNU General Public License, and \n\
 you are welcome to change it and/or distribute copies of it under \n\
 certain conditions.  There is absolutely no warranty for GP2C.\n");
@@ -308,7 +309,9 @@ user option: \n\
  -o <file> :  Place output in file <file>. \n\
  -g : Generate automatic garbage collection code. \n\
  -iN: Set indentation level to N spaces (default 2). \n\
- -W : Output information about global variables. \n\
+ -W : Output warning about types and global variables usage. \n\
+ -C : Generate range checking code. \n\
+ -L : Generate #line directive for better C compiler messages. \n\
  -p <prefix>: Prefix user-defined symbol by <prefix> to avoid conflict. \n\
  -s <suffix>: Add suffix <suffix> to GP install names of functions. \n\
  -S : Assume strict declarations for functions. \n\
@@ -337,13 +340,14 @@ int main(int argc, char **argv)
   int c;
   FILE *fin=stdin;
   FILE *fout=stdout;
-  const char *outfile = NULL, *infile = "stdin";
+  const char *infile = "stdin";
   autogc=0;
   safecoeff=0;
   warn=0;
   optprefix=NULL;
   optsuffix=NULL;
-  while((c=getopt(argc,argv,"gi:hflo:p:s:StvydCGTVW"))!=-1)
+  outfile = NULL;
+  while((c=getopt(argc,argv,"gi:hflo:p:s:StvydCGLTVW"))!=-1)
   {
     switch(c)
     {
@@ -402,6 +406,9 @@ int main(int argc, char **argv)
     case 'G':
       optgen=1-optgen;
       break;
+    case 'L':
+      opthashline=1-opthashline;
+      break;
     case 'V':
       optcleanvar=1-optcleanvar;
       break;
