@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2002  The PARI group.
+Copyright (C) 2002-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,7 +10,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include "config.h"
 #include <stdio.h>
@@ -71,16 +71,16 @@ void printnodeparens(FILE *fout, int n, int parens)
     fputc(']',fout);
     break;
   case Frange:
-    if (x!=GNOARG)
+    if (x!=GNORANGE)
     {
       printnode(fout,x);
-      if (y!=GNOARG)
+      if (y!=GNORANGE)
       {
         fputs("..",fout);
         printnode(fout,y);
       }
     }
-    else if (y!=GNOARG)
+    else if (y!=GNORANGE)
     {
       fputc('^',fout);
       printnode(fout,y);
@@ -231,6 +231,10 @@ void printnodeparens(FILE *fout, int n, int parens)
     fprintf(fout,"&");
     printnode(fout,x);
     break;
+  case Fvararg:
+    printnode(fout,x);
+    fprintf(fout,"[..]");
+    break;
   case Fcall:
     printnode(fout,x);
     fprintf(fout,"(");
